# coding=utf-8
class UserAgent(object):
    def __init__(self, is_mobile, os):
        self.is_mobile = is_mobile
        self.os = os


class UserAgentDetectMiddleware(object):
    def process_request(self, request):
        """
        检测是否是移动浏览器，然后设置is_mobile、os
        @param request:
        @type request:  django.http.HttpRequest
        @return:
        """
        user_agent = request.META['HTTP_USER_AGENT'].lower()
        is_mobile = False
        os = 'Windows'
        if 'micromessenger' in user_agent:
            is_mobile = True
            os = 'MicroChat'
        elif 'android' in user_agent:
            is_mobile = True
            os = 'Andriod'
        elif 'iphone' in user_agent:
            is_mobile = True
            os = 'iPhone/iOS'
        elif 'ipad' in user_agent:
            is_mobile = True
            os = 'iPad/iOS'
        elif 'windows phone' in user_agent:
            is_mobile = True
            os = 'Windows Phone'
        elif 'ucweb' in user_agent:
            is_mobile = True
            os = 'UCWEB'

        ua = UserAgent(is_mobile, os)
        setattr(request, 'ua', ua)



#!/usr/bin/python
#
# Copyright (c) 2011 Eran Sandler (eran@sandler.co.il),  http://eran.sandler.co.il,  http://forecastcloudy.net
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


import time
import logging
import re


class IdWorker(object):
    def __init__(self, worker_id, data_center_id):
        self.worker_id = worker_id
        self.data_center_id = data_center_id

        self.user_agent_parser = re.compile("[a-zA-Z][a-zA-Z\-0-9]*")
        self.logger = logging.getLogger("idworker")

        # stats
        self.ids_generated = 0

        # Tue, 21 Mar 2006 20:50:14.000 GMT
        self.twepoch = 1142974214000

        self.sequence = 0
        self.worker_id_bits = 5
        self.data_center_id_bits = 5
        self.max_worker_id = -1 ^ (-1 << self.worker_id_bits)
        self.max_data_center_id = -1 ^ (-1 << self.data_center_id_bits)
        self.sequence_bits = 12

        self.worker_id_shift = self.sequence_bits
        self.data_center_id_shift = self.sequence_bits + self.worker_id_bits
        self.timestamp_left_shift = self.sequence_bits + self.worker_id_bits + self.data_center_id_bits
        self.sequence_mask = -1 ^ (-1 << self.sequence_bits)

        self.last_timestamp = -1

        # Sanity check for worker_id
        if self.worker_id > self.max_worker_id or self.worker_id < 0:
            raise ValueError("worker_id", "worker id can't be greater than %i or less than 0" % self.max_worker_id)

        if self.data_center_id > self.max_data_center_id or self.data_center_id < 0:
            raise ValueError("data_center_id",
                             "data center id can't be greater than %i or less than 0" % self.max_data_center_id)

        self.logger.info(
            "worker starting. timestamp left shift %d, data center id bits %d, worker id bits %d, sequence bits %d, worker id %d" % (
                self.timestamp_left_shift, self.data_center_id_bits, self.worker_id_bits, self.sequence_bits,
                self.worker_id))

    def _time_gen(self):
        return int(time.time() * 1000)

    def _till_next_millis(self, last_timestamp):
        timestamp = self._time_gen()
        while timestamp <= last_timestamp:
            timestamp = self._time_gen()

        return timestamp

    def _next_id(self):
        timestamp = self._time_gen()

        if self.last_timestamp > timestamp:
            self.logger.warning("clock is moving backwards. Rejecting request until %i" % self.last_timestamp)
            raise ValueError("Clock moved backwards. Refusing to generate id for %i milliseocnds" % self.last_timestamp)

        if self.last_timestamp == timestamp:
            self.sequence = (self.sequence + 1) & self.sequence_mask
            if self.sequence == 0:
                timestamp = self._till_next_millis(self.last_timestamp)
        else:
            self.sequence = 0

        self.last_timestamp = timestamp

        new_id = ((timestamp - self.twepoch) << self.timestamp_left_shift) | (
            self.data_center_id << self.data_center_id_shift) | (self.worker_id << self.worker_id_shift) | self.sequence
        self.ids_generated += 1
        return new_id

    def _valid_user_agent(self, user_agent):
        return self.user_agent_parser.search(user_agent) is not None

    def get_worker_id(self):
        return self.worker_id

    def get_timestamp(self):
        return self._time_gen()

    def get_id(self, useragent):
        if not self._valid_user_agent(useragent):
            raise ValueError('Invalid useragent')

        new_id = self._next_id()
        self.logger.debug("id: %i  user_agent: %s  worker_id: %i  data_center_id: %i" % (
            new_id, useragent, self.worker_id, self.data_center_id))
        return new_id

    def get_datacenter_id(self):
        return self.data_center_id



# coding=utf-8
import json
import os
import datetime

from django.core.files.uploadhandler import TemporaryFileUploadHandler
from django.core.cache import cache
from django.utils.deconstruct import deconstructible
import shortuuid


@deconstructible
class UploadTo(object):
    def __init__(self, sub_path, fmt, rename=True):
        self.path = sub_path
        self.fmt = fmt
        self.rename = rename

    def __call__(self, instance, filename):
        ext = os.path.splitext(filename)[1]

        today = datetime.datetime.today()

        name = '%s%s' % (shortuuid.uuid(), ext.lower()) if self.rename else filename

        return os.path.join(self.path, today.strftime(self.fmt), name)


# class who handles the upload
class ProgressUploadHandler(TemporaryFileUploadHandler):
    """
    Download the file and store progression in the cache
    """

    def __init__(self, request=None):
        super(ProgressUploadHandler, self).__init__(request)
        self.progress_id = None
        self.cache_key = None
        self.request = request

    def new_file(self, file_name, *args, **kwargs):
        content_length = self.content_length
        super(ProgressUploadHandler, self).new_file(file_name, *args, **kwargs)
        if 'X-Progress-ID' in self.request.GET:
            self.progress_id = self.request.GET['X-Progress-ID']
        elif 'X-Progress-ID' in self.request.META:
            self.progress_id = self.request.META['X-Progress-ID']
        if self.progress_id:
            self.cache_key = self.progress_id
            progress_data = json.loads(cache.get('upload_progress_%s' % self.cache_key, '{}'))
            progress_data.update({
                self.field_name: {
                    'length': content_length,
                    'field_name': self.field_name,
                    'uploaded': 0
                }
            })
            cache.set('upload_progress_%s' % self.cache_key, json.dumps(progress_data))

    def handle_raw_input(self, input_data, META, content_length, boundary, encoding=None):
        self.content_length = content_length

    def receive_data_chunk(self, raw_data, start):
        if self.progress_id:
            progress_data = json.loads(cache.get('upload_progress_%s' % self.cache_key, "{}"))
            data = progress_data[self.field_name]
            data['uploaded'] += self.chunk_size
            cache.set('upload_progress_%s' % self.cache_key, json.dumps(progress_data))

        # data wont be passed to any other handler
        super(ProgressUploadHandler, self).receive_data_chunk(raw_data, start)

    def file_complete(self, file_size):
        if self.progress_id and 'upload_progress_%s' % self.cache_key in cache:
            progress_data = json.loads(cache.get('upload_progress_%s' % self.cache_key))
            progress_data.pop(self.field_name)
            if progress_data:
                cache.set('upload_progress_%s' % self.cache_key, json.dumps(progress_data))
            else:
                cache.delete('upload_progress_%s' % self.cache_key)

        return super(ProgressUploadHandler, self).file_complete(file_size)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

try:
    from django.forms.widgets import flatatt
except:
    from django.forms.utils import flatatt
from django.template import Variable, VariableDoesNotExist
from django.template.base import FilterExpression, kwarg_re, TemplateSyntaxError

from .text import text_value

# RegEx for quoted string
QUOTED_STRING = re.compile(r'^["\'](?P<noquotes>.+)["\']$')


def handle_var(value, context):
    """
    Handle template tag variable
    """
    # Resolve FilterExpression and Variable immediately
    if isinstance(value, FilterExpression) or isinstance(value, Variable):
        return value.resolve(context)
    # Return quoted strings unquoted
    # http://djangosnippets.org/snippets/886
    stringval = QUOTED_STRING.search(value)
    if stringval:
        return stringval.group('noquotes')
    # Resolve variable or return string value
    try:
        return Variable(value).resolve(context)
    except VariableDoesNotExist:
        return value


def parse_token_contents(parser, token):
    """
    Parse template tag contents
    """
    bits = token.split_contents()
    tag = bits.pop(0)
    args = []
    kwargs = {}
    asvar = None
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]
    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError(
                    'Malformed arguments to tag "{}"'.format(tag))
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))
    return {
        'tag': tag,
        'args': args,
        'kwargs': kwargs,
        'asvar': asvar,
    }


def split_css_classes(css_classes):
    """
    Turn string into a list of CSS classes
    """
    classes_list = text_value(css_classes).split(' ')
    return [c for c in classes_list if c]


def add_css_class(css_classes, css_class, prepend=False):
    """
    Add a CSS class to a string of CSS classes
    """
    classes_list = split_css_classes(css_classes)
    classes_to_add = [c for c in split_css_classes(css_class)
                      if c not in classes_list]
    if prepend:
        classes_list = classes_to_add + classes_list
    else:
        classes_list += classes_to_add
    return ' '.join(classes_list)


def remove_css_class(css_classes, css_class):
    """
    Remove a CSS class from a string of CSS classes
    """
    remove = set(split_css_classes(css_class))
    classes_list = [c for c in split_css_classes(css_classes)
                    if c not in remove]
    return ' '.join(classes_list)


def render_link_tag(url, rel='stylesheet', media='all'):
    """
    Build a link tag
    """
    return render_tag(
        'link',
        attrs={'href': url, 'rel': rel, 'media': media},
        close=False)


def render_tag(tag, attrs=None, content=None, close=True):
    """
    Render a HTML tag
    """
    builder = '<{tag}{attrs}>{content}'
    if content or close:
        builder += '</{tag}>'
    return builder.format(
        tag=tag,
        attrs=flatatt(attrs) if attrs else '',
        content=text_value(content),
    )



# coding=utf-8
import requests
from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.message import sanitize_address


class MailgunAPIError(Exception):
    pass


class MailgunBackend(BaseEmailBackend):
    """A Django Email backend that uses mailgun.
    """

    def __init__(self, fail_silently=False, *args, **kwargs):
        access_key, server_name = (kwargs.pop('access_key', None),
                                   kwargs.pop('server_name', None))

        super(MailgunBackend, self).__init__(fail_silently=fail_silently, **kwargs)

        try:
            self._access_key = access_key or getattr(settings, 'MAILGUN_ACCESS_KEY')
            self._server_name = server_name or getattr(settings, 'MAILGUN_SERVER_NAME')
        except AttributeError:
            if fail_silently:
                self._access_key, self._server_name = None
            else:
                raise

        self._api_url = "https://api.mailgun.net/v3/%s/" % self._server_name

    def open(self):
        """Stub for open connection, all sends are done over HTTP POSTs
        """
        pass

    def close(self):
        """Close any open HTTP connections to the API server.
        """
        pass

    def _send(self, email_message):
        """A helper method that does the actual sending."""
        if not email_message.recipients():
            return False
        from_email = sanitize_address(email_message.from_email, email_message.encoding)
        recipients = [sanitize_address(addr, email_message.encoding)
                      for addr in email_message.recipients()]

        try:
            r = requests. \
                post(self._api_url + "messages.mime",
                     auth=("api", self._access_key),
                     data={
                         "to": ", ".join(recipients),
                         "from": from_email,
                         'o:tracking': 'yes'
                     },
                     files={
                         "message": email_message.message().as_string(),
                     }
                     )
        except:
            if not self.fail_silently:
                raise
            return False

        if r.status_code != 200:
            if not self.fail_silently:
                raise MailgunAPIError(r)
            return False

        return True

    def send_messages(self, email_messages):
        """Sends one or more EmailMessage objects and returns the number of
        email messages sent.
        """
        if not email_messages:
            return

        num_sent = 0
        for message in email_messages:
            if self._send(message):
                num_sent += 1

        return num_sent



# -*- coding: utf-8 -*-
from django.conf import settings
from django.forms.utils import flatatt
from django.forms.widgets import DateTimeInput, TimeInput
from django.utils import translation
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape

try:
    import json
except ImportError:
    from django.utils import simplejson as json
try:
    from django.utils.encoding import force_unicode as force_text
except ImportError:  # python3
    from django.utils.encoding import force_text


def get_language():
    lang = translation.get_language()
    return 'zh-CN' if lang == 'zh-hans' else lang


class DateTimePicker(DateTimeInput):
    class Media:
        css = {'all': ('%szui/lib/datetimepicker/datetimepicker.min.css' % settings.STATIC_URL,), }
        js = ('%szui/lib/datetimepicker/datetimepicker.min.js' % settings.STATIC_URL,)

    # http://momentjs.com/docs/#/parsing/string-format/
    # http://docs.python.org/2/library/datetime.html#strftime-strptime-behavior
    format_map = (
        ('dd', r'%d'),
        ('d', r'%d'),

        ('mm', r'%m'),
        ('m', r'%m'),

        ('yyyy', r'%Y'),
        ('yy', r'%y'),

        ('MM', r'%B'),
        ('M', r'%b'),

        ('HH', r'%I'),
        ('H', r'%I'),
        ('hh', r'%H'),
        ('h', r'%H'),
        ('ii', r'%M'),
        ('i', r'%M'),

        ('ss', r'%S'),

        ('p', r'%P'),
        ('P', r'%P'),
    )

    @classmethod
    def conv_datetime_format_py2js(cls, format):
        for js, py in cls.format_map:
            format = format.replace(py, js)
        return format

    @classmethod
    def conv_datetime_format_js2py(cls, format):
        for js, py in cls.format_map:
            format = format.replace(js, py)
        return format.replace("%%", "%")  # fix some error

    html_template = '''
        <div%(div_attrs)s>
            <input%(input_attrs)s/>
            <span class="input-group-addon">
                <span%(icon_attrs)s></span>
            </span>
        </div>'''

    js_template = '''
        <script>
        $(document).ready(function(){
            $("#%(picker_id)s").datetimepicker(%(options)s);
        })
        </script>'''

    def __init__(self, attrs=None, format=None, options=None, show_icon=False, div_attrs=None, icon_attrs=None):
        if not show_icon:
            self.html_template = "<input%(input_attrs)s/>"
        self.show_icon = show_icon
        if not icon_attrs:
            icon_attrs = {'class': 'icon-calendar'}
        if not div_attrs:
            div_attrs = {'class': 'input-group date'}
        if format is None and options and options.get('format'):
            format = self.conv_datetime_format_js2py(options.get('format'))
        elif format:
            format = self.conv_datetime_format_js2py(format)
        else:
            format = self.conv_datetime_format_js2py("yyyy-mm-dd hh:ii")
        super(DateTimePicker, self).__init__(attrs, format)
        if 'class' not in self.attrs:
            self.attrs['class'] = 'form-control'
        self.div_attrs = div_attrs and div_attrs.copy() or {}
        self.icon_attrs = icon_attrs and icon_attrs.copy() or {}
        self.picker_id = self.div_attrs.get('id') or None
        if options == False:  # datetimepicker will not be initalized only when options is False
            self.options = False
        else:
            self.options = options and options.copy() or {}
            self.options['language'] = get_language()
            self.options['weekStart'] = 1
            self.options['todayBtn'] = 1
            self.options['autoclose'] = 1
            self.options['todayHighlight'] = 1
            self.options['startView'] = 2
            self.options['forceParse'] = 0
            if format and not self.options.get('format') and not self.attrs.get('date-format'):
                self.options['format'] = self.conv_datetime_format_py2js(format)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        input_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            input_attrs['value'] = force_text(self._format_value(value))
        input_attrs = dict([(key, conditional_escape(val)) for key, val in input_attrs.items()])  # python2.6 compatible
        if not self.picker_id:
            self.picker_id = input_attrs.get('id', '') + ('_picker' if self.show_icon else '')
        self.div_attrs['id'] = self.picker_id
        picker_id = conditional_escape(self.picker_id)
        div_attrs = dict(
            [(key, conditional_escape(val)) for key, val in self.div_attrs.items()])  # python2.6 compatible
        icon_attrs = dict([(key, conditional_escape(val)) for key, val in self.icon_attrs.items()])
        html = self.html_template % dict(div_attrs=flatatt(div_attrs),
                                         input_attrs=flatatt(input_attrs),
                                         icon_attrs=flatatt(icon_attrs))
        if not self.options:
            js = ''
        else:
            js = self.js_template % dict(picker_id=picker_id,
                                         options=json.dumps(self.options or {}))
        return mark_safe(force_text(html + js))


class DatePicker(DateTimePicker):
    def __init__(self, attrs=None, format=None, options=None, show_icon=False, div_attrs=None, icon_attrs=None):
        self.show_icon = show_icon
        if format is None and options and options.get('format'):
            format = options.get('format')
        else:
            format = "yyyy-mm-dd"
        options = options or {}
        options['weekStart'] = 1
        options['todayBtn'] = 1
        options['autoclose'] = 1
        options['todayHighlight'] = 1
        options['startView'] = 2
        options['minView'] = 2
        options['forceParse'] = 0
        options['format'] = format
        options['language'] = get_language()
        super(DatePicker, self).__init__(attrs=attrs, format=format, options=options, show_icon=show_icon,
                                         div_attrs=div_attrs,
                                         icon_attrs=icon_attrs)


# As a TimeInput

class TimePicker(TimeInput):
    class Media:
        css = {'all': ('zui/lib/datetimepicker/datetimepicker.min.css',), }
        js = ('%szui/lib/datetimepicker/datetimepicker.min.js' % settings.STATIC_URL,)

    # http://www.malot.fr/bootstrap-datetimepicker/
    # http://docs.python.org/2/library/datetime.html#strftime-strptime-behavior
    format_map = (('HH', r'%H'),
                  ('hh', r'%I'),
                  ('mm', r'%M'),
                  ('ss', r'%S'),
                  ('a', r'%p'),
                  ('ZZ', r'%z'),
                  )

    @classmethod
    def conv_datetime_format_py2js(cls, format):
        for js, py in cls.format_map:
            format = format.replace(py, js)
        return format

    @classmethod
    def conv_datetime_format_js2py(cls, format):
        for js, py in cls.format_map:
            format = format.replace(js, py)
        return format

    html_template = '''
        <div%(div_attrs)s>
            <input%(input_attrs)s/>
            <span class="input-group-addon">
                <span%(icon_attrs)s></span>
            </span>
        </div>'''

    js_template = '''
        <script>
            $(document).ready(function(){
                    $("#%(picker_id)s").datetimepicker(%(options)s);
            });
        </script>'''

    def __init__(self, attrs=None, format=None, options=None, show_icon=False, div_attrs=None, icon_attrs=None):
        self.show_icon = show_icon
        if not show_icon:
            self.html_template = "<input%(input_attrs)s/>"
        if not icon_attrs:
            icon_attrs = {'class': 'icon-time'}
        if not div_attrs:
            div_attrs = {'class': 'input-group date'}
        if format is None and options and options.get('format'):
            format = self.conv_datetime_format_js2py(options.get('format'))
        elif format:
            format = self.conv_datetime_format_js2py(format)
        else:
            format = self.conv_datetime_format_js2py("hh:ii")
        super(TimePicker, self).__init__(attrs, format)
        if 'class' not in self.attrs:
            self.attrs['class'] = 'form-control'
        self.div_attrs = div_attrs and div_attrs.copy() or {}
        self.icon_attrs = icon_attrs and icon_attrs.copy() or {}
        self.picker_id = self.div_attrs.get('id') or None
        if options == False:  # datetimepicker will not be initalized only when options is False
            self.options = False
        else:
            self.options = options and options.copy() or {}
            self.options['language'] = get_language()
            self.options['weekStart'] = 1
            self.options['todayBtn'] = 1
            self.options['autoclose'] = 1
            self.options['todayHighlight'] = 1
            self.options['startView'] = 1
            self.options['minView'] = 0
            self.options['maxView'] = 1
            self.options['forceParse'] = 0
            if format and not self.options.get('format') and not self.attrs.get('date-format'):
                self.options['format'] = self.conv_datetime_format_py2js(format)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        input_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            input_attrs['value'] = force_text(self._format_value(value))
        input_attrs = dict([(key, conditional_escape(val)) for key, val in input_attrs.items()])  # python2.6 compatible
        if not self.picker_id:
            self.picker_id = input_attrs.get('id', '') + ('_picker' if self.show_icon else '')
        self.div_attrs['id'] = self.picker_id
        picker_id = conditional_escape(self.picker_id)
        div_attrs = dict(
            [(key, conditional_escape(val)) for key, val in self.div_attrs.items()])  # python2.6 compatible
        icon_attrs = dict([(key, conditional_escape(val)) for key, val in self.icon_attrs.items()])
        html = self.html_template % dict(div_attrs=flatatt(div_attrs),
                                         input_attrs=flatatt(input_attrs),
                                         icon_attrs=flatatt(icon_attrs))
        if not self.options:
            js = ''
        else:
            js = self.js_template % dict(picker_id=picker_id,
                                         options=json.dumps(self.options or {}))
        return mark_safe(force_text(html + js))



# coding=utf-8
from warnings import warn

from django.utils.encoding import force_text


class BaseBV(object):
    code = ''
    message = None

    def __call__(self, value):
        # value = force_text(value)
        # TODO: support server side validation
        warn("django-scaffold-toolkit.formValidator.%s not support server side validation now." % self.code)
        return

    def get_validator_code(self):
        return {}

    def _patch_message(self, context):
        if self.message:
            context['message'] = self.message
        return context


class IdValidator(BaseBV):
    """
    Validate identification number
    http://formvalidation.io/validators/id/
    """
    code = 'id'
    COUNTRIES = {'BA': 'Bosnia and Herzegovina',
                 'BG': 'Bulgaria',
                 'BR': 'Brazil',
                 'CH': 'Switzerland',
                 'CL': 'Chile',
                 'CN': 'China',
                 'CZ': 'Czech Republic',
                 'DK': 'Denmark',
                 'EE': 'Estonia',
                 'ES': 'Spain',
                 'FI': 'Finland',
                 'HR': 'Croatia',
                 'IE': 'Ireland',
                 'IS': 'Iceland',
                 'LT': 'Lithuania',
                 'LV': 'Latvia',
                 'ME': 'Montenegro',
                 'MK': 'Macedonia',
                 'NL': 'Netherlands',
                 'RO': 'Romania',
                 'RS': 'Serbia',
                 'SE': 'Sweden',
                 'SI': 'Slovenia',
                 'SK': 'Slovakia',
                 'SM': 'San Marino',
                 'TH': 'Thailand',
                 'ZA': 'South Africa'}

    def __init__(self, country, message=None):
        """

        :param country: 可以是国家代码，或者是另外一个控件名字的引用
        :return:
        """
        self.country = country
        self.message = message

    def get_validator_code(self):
        return {self.code: self._patch_message({'country': self.country})}


class ZipCodeValidator(BaseBV):
    """
    :param country: 可以是国家代码，或者是另外一个控件名字的引用
    """
    code = 'zipZode'
    COUNTRIES = {
        'BR': 'Brazil',
        'CA': 'Canada',
        'CZ': 'Czech Republic',
        'DK': 'Denmark',
        'GB': 'United Kingdom',
        'IT': 'Italy',
        'MA': 'Morocco',
        'NL': 'Netherlands',
        'RO': 'Romania',
        'RU': 'Russia',
        'SE': 'Sweden',
        'SG': 'Singapore',
        'SK': 'Slovakia',
        'US': 'USA'
    }

    def __init__(self, country, message=None):
        self.country = country
        self.message = message

    def __call__(self, value):
        # value = force_text(value)
        # TODO: support server side validation
        warn("Bootstrap_Validator.zipCode not support server side validation now.")
        return

    def get_validator_code(self):
        return {self.code: self._patch_message({'country': self.country})}


class IdenticalValidator(BaseBV):
    """
    Check if the value is the same as one of particular field

    http://formvalidation.io/validators/identical/
    """
    code = 'identical'

    def __init__(self, field, message=None):
        self.field = field
        self.message = message

    def get_validator_code(self):
        return {self.code: self._patch_message({'field': self.field})}


class DifferentValidator(BaseBV):
    """
    Return true if the input value is different with given field's value

    http://formvalidation.io/validators/diffenert/
    """
    code = 'different'

    def __init__(self, field, message=None):
        self.field = field
        self.message = message

    def get_validator_code(self):
        return {self.code: self._patch_message({'field': self.field})}


class RemoteValidator(BaseBV):
    """
    Perform remote checking via Ajax request

    http://formvalidation.io/validators/remote/
    """

    code = 'remote'

    def __init__(self, url, type='GET', name=None, data=None, delay=None, message=None):
        self.url = url
        self.type = type
        self.name = name
        self.data = data
        self.delay = delay
        self.message = message

    def get_validator_code(self):
        vc = {'url': force_text(self.url),  # url maybe lazy,
              'type': self.type}
        if self.name:
            vc['name'] = self.name
        if self.data:
            vc['data'] = self.data
        if self.delay:
            vc['delay'] = self.delay
        else:
            vc['delay'] = 300
        return {'remote': self._patch_message(vc)}


class ChoicesValidator(BaseBV):
    """
    Check if the number of checked boxes are less or more than a given number

    http://formvalidation.io/validators/choice/
    """
    code = 'choices'

    def __init__(self, _min, _max, message=None):
        self.min = _min
        self.max = _max
        self.message = message

    def get_validator_code(self):
        return {self.code: self._patch_message({'min': self.min, 'max': self.max})}


class CallBackValidator(BaseBV):
    """
    Return the validity from a callback method

    http://formvalidation.io/validators/callback/
    """
    code = 'callback'

    def __init__(self, callback, message=None):
        """

        :param callback: the callback code or function name
        :return:
        """
        self.callback = callback
        self.message = message

    def get_validator_code(self):
        return {self.callback: self._patch_message({'callback': self.callback})}


class PhoneValidator(BaseBV):
    """
    Validate a phone number

    http://formvalidation.io/validators/phone/
    """
    code = 'phone'
    COUNTRIES = {
        'BR': 'Brazil',
        'CN': 'China',
        'CZ': 'Czech Republic',
        'DK': 'Denmark',
        'ES': 'Spain',
        'FR': 'France',
        'GB': 'United Kingdom',
        'MA': 'Morocco',
        'PK': 'Pakistan',
        'RO': 'Romania',
        'RU': 'Russia',
        'SK': 'Slovakia',
        'TH': 'Thailand',
        'US': 'USA',
        'VE': 'Venezuela'
    }

    def __init__(self, country, message=None):
        self.country = country

    def get_validator_code(self):
        return {self.code: self._patch_message({'country': self.country})}


class EmailAddressValidator(BaseBV):
    code = 'emailAddress'

    def __init__(self, multiple=False, separator='/[,;]/', message=None):
        self.multiple = multiple
        self.separator = separator
        self.message = message

    def get_validator_code(self):
        return {self.code: self._patch_message({'multiple': str(self.multiple).lower(),
                                                'separator': self.separator})}


class UriValidator(BaseBV):
    code = 'uri'

    def __init__(self, allowLocal=False, protocol='http,https,ftp', message=None):
        self.allowLocal = allowLocal
        self.protocol = protocol
        self.message = message

    def get_validator_code(self):
        return {self.code: self._patch_message({'allowLocal': str(self.allowLocal).lower(),
                                                'protocol': self.protocol})}


class FileValidator(BaseBV):
    """
    Validate file

    :Warn The maxSize and type are only used if the browser supports HTML 5 File API.

    http://formvalidation.io/validators/file/
    """
    code = 'file'

    def __init__(self, extension, type=None, minSize=None, maxSize=None, message=None):
        self.extension = extension
        self.type = type
        self.minSize = minSize
        self.maxSize = maxSize
        self.message = message

    def get_validator_code(self):
        vc = {'extension': self.extension}
        if self.type:
            vc['type'] = self.type
        if self.minSize:
            vc['minSize'] = self.minSize
        if self.maxSize:
            vc['maxSize'] = self.maxSize

        return {self.code: self._patch_message(vc)}


class ImageFileValidator(FileValidator):
    def __init__(self, minSize=None, maxSize=None):
        super(ImageFileValidator, self).__init__('jpg,jpeg,png,bmp,gif,webp,ico,jnp',
                                                 'image/jpeg,image/png,images/gif,images/x-icon,images/x-ms-bmp,images/webp,images/x-jnp',
                                                 minSize, maxSize)


class VideoFileValidator(FileValidator):
    def __init__(self, minSize=None, maxSize=None):
        super(VideoFileValidator, self).__init__('mp4,3gp,3gpp,megp,mgp,mov,webm,flv,m4v,wmv,avi,',
                                                 'video/mp4,video/3gpp,video/mpeg,video/quicktime,video/x-flv,video/x-m4v,video/x-ms-wmv,video/x-msvideo,video/webm',
                                                 minSize, maxSize)


class AudioFileValidator(FileValidator):
    def __init__(self, minSize=None, maxSize=None):
        super(AudioFileValidator, self).__init__('mid,midi,kar,mp3,ogg,m4a,ra',
                                                 'audio/midi,audio/mpeg,audio/ogg,audio/x-m4a,audio/x-realaudio')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from math import floor

from django import template
from django.template.loader import get_template

from ..zui import (
    css_url, javascript_url, jquery_url, theme_url, get_zui_setting
)
from ..utils import render_link_tag
from ..forms import (
    render_button, render_field, render_field_and_label, render_form,
    render_form_group, render_formset,
    render_label, render_form_errors, render_formset_errors
)
from ..components import render_icon, render_alert
from ..utils import handle_var, parse_token_contents
from ..text import force_text

register = template.Library()


@register.filter
def zui_setting(value):
    """
    A simple way to read zui settings in a template.
    Please consider this filter private for now, do not use it in your own
    templates.
    """
    return get_zui_setting(value)


@register.simple_tag
def zui_jquery_url():
    """
    **Tag name**::

        zui_jquery_url

    Return the full url to jQuery file to use

    Default value: ``//code.jquery.com/jquery.min.js``

    This value is configurable, see Settings section

    **usage**::

        {% zui_jquery_url %}

    **example**::

        {% zui_jquery_url %}
    """
    return jquery_url()


@register.simple_tag
def zui_javascript_url():
    """
    Return the full url to the zui JavaScript library

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        zui_javascript_url

    **usage**::

        {% zui_javascript_url %}

    **example**::

        {% zui_javascript_url %}
    """
    return javascript_url()


@register.simple_tag
def zui_css_url():
    """
    Return the full url to the zui CSS library

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        zui_css_url

    **usage**::

        {% zui_css_url %}

    **example**::

        {% zui_css_url %}
    """
    return css_url()


@register.simple_tag
def zui_theme_url():
    """
    Return the full url to a zui theme CSS library

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        zui_css_url

    **usage**::

        {% zui_css_url %}

    **example**::

        {% zui_css_url %}
    """
    return theme_url()


@register.simple_tag
def zui_css():
    """
    Return HTML for zui CSS
    Adjust url in settings. If no url is returned, we don't want this statement
    to return any HTML.
    This is intended behavior.

    Default value: ``FIXTHIS``

    This value is configurable, see Settings section

    **Tag name**::

        zui_css

    **usage**::

        {% zui_css %}

    **example**::

        {% zui_css %}
    """
    urls = [url for url in [zui_css_url(), zui_theme_url()] if url]
    return ''.join([render_link_tag(url) for url in urls])


@register.simple_tag
def zui_javascript(jquery=None):
    """
    Return HTML for zui JavaScript.

    Adjust url in settings. If no url is returned, we don't want this
    statement to return any HTML.
    This is intended behavior.

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        zui_javascript

    **Parameters**:

        :jquery: Truthy to include jQuery as well as zui

    **usage**::

        {% zui_javascript %}

    **example**::

        {% zui_javascript jquery=1 %}
    """

    javascript = ''
    # See if we have to include jQuery
    if jquery is None:
        jquery = get_zui_setting('include_jquery', False)
    # NOTE: No async on scripts, not mature enough. See issue #52 and #56
    if jquery:
        url = zui_jquery_url()
        if url:
            javascript += '<script src="{url}"></script>'.format(url=url)
    url = zui_javascript_url()
    if url:
        javascript += '<script src="{url}"></script>'.format(url=url)
    return javascript


@register.simple_tag
def zui_formset(*args, **kwargs):
    """
    Render a formset


    **Tag name**::

        zui_formset

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_formset formset %}

    **example**::

        {% zui_formset formset layout='horizontal' %}

    """
    return render_formset(*args, **kwargs)


@register.simple_tag
def zui_formset_errors(*args, **kwargs):
    """
    Render formset errors

    **Tag name**::

        zui_formset_errors

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_formset_errors formset %}

    **example**::

        {% zui_formset_errors formset layout='inline' %}
    """
    return render_formset_errors(*args, **kwargs)


@register.simple_tag
def zui_form(*args, **kwargs):
    """
    Render a form

    **Tag name**::

        zui_form

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_form form %}

    **example**::

        {% zui_form form layout='inline' %}
    """
    return render_form(*args, **kwargs)


@register.simple_tag
def zui_form_errors(*args, **kwargs):
    """
    Render form errors

    **Tag name**::

        zui_form_errors

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_form_errors form %}

    **example**::

        {% zui_form_errors form layout='inline' %}
    """
    return render_form_errors(*args, **kwargs)


@register.simple_tag
def zui_field(*args, **kwargs):
    """
    Render a field

    **Tag name**::

        zui_field

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_field form_field %}

    **example**::

        {% zui_field form_field %}
    """
    return render_field(*args, **kwargs)


@register.simple_tag()
def zui_label(*args, **kwargs):
    """
    Render a label

    **Tag name**::

        zui_label

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_label FIXTHIS %}

    **example**::

        {% zui_label FIXTHIS %}
    """
    return render_label(*args, **kwargs)


@register.simple_tag
def zui_button(*args, **kwargs):
    """
    Render a button

    **Tag name**::

        zui_button

    **Parameters**:

        :args:
        :kwargs:

    **usage**::

        {% zui_button FIXTHIS %}

    **example**::

        {% zui_button FIXTHIS %}
    """
    return render_button(*args, **kwargs)


@register.simple_tag
def zui_icon(icon, **kwargs):
    """
    Render an icon

    **Tag name**::

        zui_icon

    **Parameters**:

        :icon: icon name

    **usage**::

        {% zui_icon "icon_name" %}

    **example**::

        {% zui_icon "star" %}

    """
    return render_icon(icon, **kwargs)


@register.simple_tag
def zui_alert(content, alert_type='info', dismissable=True):
    """
    Render an alert

    **Tag name**::

        zui_alert

    **Parameters**:

        :content: HTML content of alert
        :alert_type: one of 'info', 'warning', 'danger' or 'success'
        :dismissable: boolean, is alert dismissable

    **usage**::

        {% zui_alert "my_content" %}

    **example**::

        {% zui_alert "Something went wrong" alert_type='error' %}

    """
    return render_alert(content, alert_type, dismissable)


@register.tag('buttons')
def zui_buttons(parser, token):
    """
    Render buttons for form

    **Tag name**::

        zui_buttons

    **Parameters**:

        :parser:
        :token:

    **usage**::

        {% zui_buttons FIXTHIS %}

    **example**::

        {% zui_buttons FIXTHIS %}
    """
    kwargs = parse_token_contents(parser, token)
    kwargs['nodelist'] = parser.parse(('endbuttons',))
    parser.delete_first_token()
    return ButtonsNode(**kwargs)


class ButtonsNode(template.Node):
    def __init__(self, nodelist, args, kwargs, asvar, **kwargs2):
        self.nodelist = nodelist
        self.args = args
        self.kwargs = kwargs
        self.asvar = asvar

    def render(self, context):
        output_kwargs = {}
        for key in self.kwargs:
            output_kwargs[key] = handle_var(self.kwargs[key], context)
        buttons = []
        submit = output_kwargs.get('submit', None)
        reset = output_kwargs.get('reset', None)
        if submit:
            buttons.append(zui_button(submit, 'submit'))
        if reset:
            buttons.append(zui_button(reset, 'reset'))
        buttons = ' '.join(buttons) + self.nodelist.render(context)
        output_kwargs.update({
            'label': None,
            'field': buttons,
        })
        output = render_form_group(render_field_and_label(**output_kwargs))
        if self.asvar:
            context[self.asvar] = output
            return ''
        else:
            return output


@register.simple_tag(takes_context=True)
def zui_messages(context, *args, **kwargs):
    """
    Show django.contrib.messages Messages in zui alert containers.

    In order to make the alerts dismissable (with the close button),
    we have to set the jquery parameter too when using the
    zui_javascript tag.

    **Tag name**::

        zui_messages

    **Parameters**:

        :context:
        :args:
        :kwargs:

    **usage**::

        {% zui_messages FIXTHIS %}

    **example**::

        {% zui_javascript jquery=1 %}
        {% zui_messages FIXTHIS %}

    """
    return get_template('zui/messages.html').render({'messages': context['messages']})


@register.inclusion_tag('zui/zui_pagination.html')
def zui_pagination(page, **kwargs):
    """
    Render pagination for a page

    **Tag name**::

        zui_pagination

    **Parameters**:

        :page:
        :parameter_name: Name of paging URL parameter (default: "page")
        :kwargs:

    **usage**::

        {% zui_pagination FIXTHIS %}

    **example**::

        {% zui_pagination FIXTHIS %}
    """

    pagination_kwargs = kwargs.copy()
    pagination_kwargs['page'] = page
    return get_pagination_context(**pagination_kwargs)


def get_pagination_context(page, pages_to_show=11,
                           url=None, size=None, extra=None,
                           parameter_name='page'):
    """
    Generate zui pagination context from a page object
    """
    pages_to_show = int(pages_to_show)
    if pages_to_show < 1:
        raise ValueError("Pagination pages_to_show should be a positive " +
                         "integer, you specified {pages}".format(
                             pages=pages_to_show))
    num_pages = page.paginator.num_pages
    current_page = page.number
    half_page_num = int(floor(pages_to_show / 2)) - 1
    if half_page_num < 0:
        half_page_num = 0
    first_page = current_page - half_page_num
    if first_page <= 1:
        first_page = 1
    if first_page > 1:
        pages_back = first_page - half_page_num
        if pages_back < 1:
            pages_back = 1
    else:
        pages_back = None
    last_page = first_page + pages_to_show - 1
    if pages_back is None:
        last_page += 1
    if last_page > num_pages:
        last_page = num_pages
    if last_page < num_pages:
        pages_forward = last_page + half_page_num
        if pages_forward > num_pages:
            pages_forward = num_pages
    else:
        pages_forward = None
        if first_page > 1:
            first_page -= 1
        if pages_back is not None and pages_back > 1:
            pages_back -= 1
        else:
            pages_back = None
    pages_shown = []
    for i in range(first_page, last_page + 1):
        pages_shown.append(i)
        # Append proper character to url
    if url:
        # Remove existing page GET parameters
        url = force_text(url)
        url = re.sub(r'\?{0}\=[^\&]+'.format(parameter_name), '?', url)
        url = re.sub(r'\&{0}\=[^\&]+'.format(parameter_name), '', url)
        # Append proper separator
        if '?' in url:
            url += '&'
        else:
            url += '?'
            # Append extra string to url
    if extra:
        if not url:
            url = '?'
        url += force_text(extra) + '&'
    if url:
        url = url.replace('?&', '?')
    # Set CSS classes, see http://getzui.com/components/#pagination
    pagination_css_classes = ['paper']
    # if size == 'small':
    #     pagination_css_classes.append('pagination-sm')
    # elif size == 'large':
    #     pagination_css_classes.append('pagination-lg')
    #     # Build context object
    return {
        'zui_pagination_url': url,
        'num_pages': num_pages,
        'current_page': current_page,
        'first_page': first_page,
        'last_page': last_page,
        'pages_shown': pages_shown,
        'pages_back': pages_back,
        'pages_forward': pages_forward,
        'pagination_css_classes': ' '.join(pagination_css_classes),
        'parameter_name': parameter_name,
    }


@register.simple_tag
def zui_static(field_name, field_value, label_css_class='col-md-3', field_css_field='col-md-9'):
    """

    :param field_name:
    :param field_value:
    :param label_css_class:
    :param field_css_field:
    :return:
    """

    tmp = u"""
    <div class="form-group">
    <label class="{label_css_class} control-label">{field_name}</label>
    <div class="{field_css_class}">
      <p class="form-control-static">{field_value}</p>
    </div>
  </div>
    """
    if isinstance(field_value, bool):
        field_value = u'是' if field_value else u'否'
    else:
        field_value = field_value if field_value else u''
    return tmp.format(**{'label_css_class': label_css_class,
                         'field_name': field_name,
                         'field_css_class': field_css_field,
                         'field_value': field_value})



# coding=utf-8
from django.db import models

class TimeStampedModel(models.Model):
    created_at = models.DateTimeField(u"创建时间", auto_now_add=True, null=True, blank=True, editable=False)
    modified_at = models.DateTimeField(u"修改时间", auto_now=True, null=True, blank=True, editable=False)

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.test import TestCase

from django import forms
from django.forms.formsets import formset_factory
from django.template import Template, Context

from .text import text_value, text_concat
from .exceptions import BootstrapError
from .utils import add_css_class


RADIO_CHOICES = (
    ('1', 'Radio 1'),
    ('2', 'Radio 2'),
)

MEDIA_CHOICES = (
    ('Audio', (
        ('vinyl', 'Vinyl'),
        ('cd', 'CD'),
    )
    ),
    ('Video', (
        ('vhs', 'VHS Tape'),
        ('dvd', 'DVD'),
    )
    ),
    ('unknown', 'Unknown'),
)


class TestForm(forms.Form):
    """
    Form with a variety of widgets to test bootstrap3 rendering.
    """
    date = forms.DateField(required=False)
    subject = forms.CharField(
        max_length=100,
        help_text='my_help_text',
        required=True,
        widget=forms.TextInput(attrs={'placeholder': 'placeholdertest'}),
    )
    password = forms.CharField(widget=forms.PasswordInput)
    message = forms.CharField(required=False, help_text='<i>my_help_text</i>')
    sender = forms.EmailField(label='Sender © unicode')
    secret = forms.CharField(initial=42, widget=forms.HiddenInput)
    cc_myself = forms.BooleanField(
        required=False, help_text='You will get a copy in your mailbox.')
    select1 = forms.ChoiceField(choices=RADIO_CHOICES)
    select2 = forms.MultipleChoiceField(
        choices=RADIO_CHOICES,
        help_text='Check as many as you like.',
    )
    select3 = forms.ChoiceField(choices=MEDIA_CHOICES)
    select4 = forms.MultipleChoiceField(
        choices=MEDIA_CHOICES,
        help_text='Check as many as you like.',
    )
    category1 = forms.ChoiceField(
        choices=RADIO_CHOICES, widget=forms.RadioSelect)
    category2 = forms.MultipleChoiceField(
        choices=RADIO_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        help_text='Check as many as you like.',
    )
    category3 = forms.ChoiceField(
        widget=forms.RadioSelect, choices=MEDIA_CHOICES)
    category4 = forms.MultipleChoiceField(
        choices=MEDIA_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        help_text='Check as many as you like.',
    )
    addon = forms.CharField(
        widget=forms.TextInput(attrs={'addon_before': 'before', 'addon_after': 'after'}),
    )

    required_css_class = 'bootstrap3-req'

    def clean(self):
        cleaned_data = super(TestForm, self).clean()
        raise forms.ValidationError(
            "This error was added to show the non field errors styling.")
        return cleaned_data


class TestFormWithoutRequiredClass(TestForm):
    required_css_class = ''


def render_template(text, **context_args):
    """
    Create a template ``text`` that first loads bootstrap3.
    """
    template = Template("{% load bootstrap3 %}" + text)
    if 'form' not in context_args:
        context_args['form'] = TestForm()
    return template.render(Context(context_args))


def render_formset(formset=None, **context_args):
    """
    Create a template that renders a formset
    """
    context_args['formset'] = formset
    return render_template('{% bootstrap_formset formset %}', **context_args)


def render_form(form=None, **context_args):
    """
    Create a template that renders a form
    """
    if form:
        context_args['form'] = form
    return render_template('{% bootstrap_form form %}', **context_args)


def render_form_field(field, **context_args):
    """
    Create a template that renders a field
    """
    form_field = 'form.%s' % field
    return render_template(
        '{% bootstrap_field ' + form_field + ' %}', **context_args)


def render_field(field, **context_args):
    """
    Create a template that renders a field
    """
    context_args['field'] = field
    return render_template('{% bootstrap_field field %}', **context_args)


class SettingsTest(TestCase):
    def test_settings(self):
        from .zui import ZUI
        self.assertTrue(ZUI)

    def test_settings_filter(self):
        res = render_template(
            '{% load bootstrap3 %}' +
            '{{ "required_css_class"|bootstrap_setting }}')
        self.assertEqual(res.strip(), 'bootstrap3-req')
        res = render_template(
            '{% load bootstrap3 %}' +
            '{% if "javascript_in_head"|bootstrap_setting %}' +
            'head{% else %}body{% endif %}'
        )
        self.assertEqual(res.strip(), 'head')

    def test_required_class(self):
        form = TestForm()
        res = render_template('{% bootstrap_form form %}', form=form)
        self.assertIn('bootstrap3-req', res)

    def test_error_class(self):
        form = TestForm({})
        res = render_template('{% bootstrap_form form %}', form=form)
        self.assertIn('bootstrap3-err', res)

    def test_bound_class(self):
        form = TestForm({'sender': 'sender'})
        res = render_template('{% bootstrap_form form %}', form=form)
        self.assertIn('bootstrap3-bound', res)


class TemplateTest(TestCase):
    def test_empty_template(self):
        res = render_template('')
        self.assertEqual(res.strip(), '')

    def test_text_template(self):
        res = render_template('some text')
        self.assertEqual(res.strip(), 'some text')

    def test_bootstrap_template(self):
        template = Template((
            '{% extends "bootstrap3/bootstrap3.html" %}' +
            '{% block bootstrap3_content %}' +
            'test_bootstrap3_content' +
            '{% endblock %}'
        ))
        res = template.render(Context({}))
        self.assertIn('test_bootstrap3_content', res)

    def test_javascript_without_jquery(self):
        res = render_template('{% bootstrap_javascript jquery=0 %}')
        self.assertIn('bootstrap', res)
        self.assertNotIn('jquery', res)

    def test_javascript_with_jquery(self):
        res = render_template('{% bootstrap_javascript jquery=1 %}')
        self.assertIn('bootstrap', res)
        self.assertIn('jquery', res)


class FormSetTest(TestCase):
    def test_illegal_formset(self):
        with self.assertRaises(BootstrapError):
            render_formset(formset='illegal')


class FormTest(TestCase):
    def test_illegal_form(self):
        with self.assertRaises(BootstrapError):
            render_form(form='illegal')

    def test_field_names(self):
        form = TestForm()
        res = render_form(form)
        for field in form:
            self.assertIn('name="%s"' % field.name, res)

    def test_field_addons(self):
        form = TestForm()
        res = render_form(form)
        self.assertIn('<div class="input-group"><span class="input-group-addon">before</span><input', res)
        self.assertIn('/><span class="input-group-addon">after</span></div>', res)

    def test_exclude(self):
        form = TestForm()
        res = render_template(
            '{% bootstrap_form form exclude="cc_myself" %}', form=form)
        self.assertNotIn('cc_myself', res)

    def test_layout_horizontal(self):
        form = TestForm()
        res = render_template(
            '{% bootstrap_form form layout="horizontal" %}', form=form)
        self.assertIn('col-md-3', res)
        self.assertIn('col-md-9', res)
        res = render_template(
            '{% bootstrap_form form layout="horizontal" ' +
            'horizontal_label_class="hlabel" ' +
            'horizontal_field_class="hfield" %}',
            form=form
        )
        self.assertIn('hlabel', res)
        self.assertIn('hfield', res)

    def test_buttons_tag(self):
        form = TestForm()
        res = render_template(
            '{% buttons layout="horizontal" %}{% endbuttons %}', form=form)
        self.assertIn('col-md-3', res)
        self.assertIn('col-md-9', res)


class FieldTest(TestCase):
    def test_illegal_field(self):
        with self.assertRaises(BootstrapError):
            render_field(field='illegal')

    def test_show_help(self):
        res = render_form_field('subject')
        self.assertIn('my_help_text', res)
        self.assertNotIn('<i>my_help_text</i>', res)
        res = render_template('{% bootstrap_field form.subject show_help=0 %}')
        self.assertNotIn('my_help_text', res)

    def test_subject(self):
        res = render_form_field('subject')
        self.assertIn('type="text"', res)
        self.assertIn('placeholder="placeholdertest"', res)

    def test_password(self):
        res = render_form_field('password')
        self.assertIn('type="password"', res)
        self.assertIn('placeholder="Password"', res)

    def test_required_field(self):
        required_field = render_form_field('subject')
        self.assertIn('required', required_field)
        self.assertIn('bootstrap3-req', required_field)
        not_required_field = render_form_field('message')
        self.assertNotIn('required', not_required_field)
        # Required field with required=0
        form_field = 'form.subject'
        rendered = render_template(
            '{% bootstrap_field ' + form_field + ' set_required=0 %}')
        self.assertNotIn('required', rendered)
        # Required settings in field
        form_field = 'form.subject'
        rendered = render_template(
            '{% bootstrap_field ' +
            form_field +
            ' required_css_class="test-required" %}')
        self.assertIn('test-required', rendered)

    def test_empty_permitted(self):
        form = TestForm()
        res = render_form_field('subject', form=form)
        self.assertIn('required', res)
        form.empty_permitted = True
        res = render_form_field('subject', form=form)
        self.assertNotIn('required', res)

    def test_input_group(self):
        res = render_template(
            '{% bootstrap_field form.subject addon_before="$" ' +
            'addon_after=".00" %}'
        )
        self.assertIn('class="input-group"', res)
        self.assertIn('class="input-group-addon">$', res)
        self.assertIn('class="input-group-addon">.00', res)

    def test_size(self):
        def _test_size(param, klass):
            res = render_template(
                '{% bootstrap_field form.subject size="' + param + '" %}')
            self.assertIn(klass, res)

        def _test_size_medium(param):
            res = render_template(
                '{% bootstrap_field form.subject size="' + param + '" %}')
            self.assertNotIn('input-lg', res)
            self.assertNotIn('input-sm', res)
            self.assertNotIn('input-md', res)
        _test_size('sm', 'input-sm')
        _test_size('small', 'input-sm')
        _test_size('lg', 'input-lg')
        _test_size('large', 'input-lg')
        _test_size_medium('md')
        _test_size_medium('medium')
        _test_size_medium('')


class ComponentsTest(TestCase):
    def test_icon(self):
        res = render_template('{% bootstrap_icon "star" %}')
        self.assertEqual(
            res.strip(), '<span class="glyphicon glyphicon-star"></span>')
        res = render_template(
            '{% bootstrap_icon "star" title="alpha centauri" %}')
        self.assertEqual(
            res.strip(),
            '<span class="glyphicon glyphicon-star" ' +
            'title="alpha centauri"></span>')

    def test_alert(self):
        res = render_template(
            '{% bootstrap_alert "content" alert_type="danger" %}')
        self.assertEqual(
            res.strip(),
            '<div class="alert alert-danger alert-dismissable">' +
            '<button type="button" class="close" data-dismiss="alert" ' +
            'aria-hidden="true">' +
            '&times;</button>content</div>'
        )


class MessagesTest(TestCase):
    def test_messages(self):
        class FakeMessage(object):
            """
            Follows the `django.contrib.messages.storage.base.Message` API.
            """

            def __init__(self, message, tags):
                self.tags = tags
                self.message = message

            def __str__(self):
                return self.message

        pattern = re.compile(r'\s+')
        messages = [FakeMessage("hello", "warning")]
        res = render_template(
            '{% bootstrap_messages messages %}', messages=messages)
        expected = """
    <div class="alert alert-warning alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello
    </div>
"""
        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )

        messages = [FakeMessage("hello", "error")]
        res = render_template(
            '{% bootstrap_messages messages %}', messages=messages)
        expected = """
    <div class="alert alert-danger alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello
    </div>
        """
        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )

        messages = [FakeMessage("hello", None)]
        res = render_template(
            '{% bootstrap_messages messages %}', messages=messages)
        expected = """
    <div class="alert alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello
    </div>
        """

        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )


class TextTest(TestCase):
    def test_add_css_class(self):
        css_classes = "one two"
        css_class = "three four"
        classes = add_css_class(css_classes, css_class)
        self.assertEqual(classes, "one two three four")

        classes = add_css_class(css_classes, css_class, prepend=True)
        self.assertEqual(classes, "three four one two")


class HtmlTest(TestCase):
    def test_text_value(self):
        self.assertEqual(text_value(''), "")
        self.assertEqual(text_value(' '), " ")
        self.assertEqual(text_value(None), "")
        self.assertEqual(text_value(1), "1")

    def test_text_concat(self):
        self.assertEqual(text_concat(1, 2), "12")
        self.assertEqual(text_concat(1, 2, separator='='), "1=2")
        self.assertEqual(text_concat(None, 2, separator='='), "2")


class ButtonTest(TestCase):
    def test_button(self):
        res = render_template(
            "{% bootstrap_button 'button' size='lg' %}")
        self.assertEqual(
            res.strip(), '<button class="btn btn-lg">button</button>')
        res = render_template(
            "{% bootstrap_button 'button' size='lg' href='#' %}")
        self.assertIn(
            res.strip(),
            '<a class="btn btn-lg" href="#">button</a><a href="#" ' +
            'class="btn btn-lg">button</a>')


class ShowLabelTest(TestCase):
    def test_show_label(self):
        form = TestForm()
        res = render_template(
            '{% bootstrap_form form show_label=False %}',
            form=form
        )
        self.assertIn('sr-only', res)

    def test_for_formset(self):
        TestFormSet = formset_factory(TestForm, extra=1)
        test_formset = TestFormSet()
        res = render_template(
            '{% bootstrap_formset formset show_label=False %}',
            formset=test_formset
        )
        self.assertIn('sr-only', res)



# coding=utf-8
from django import forms
from django.shortcuts import resolve_url
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe


class TagAutocompleteInput(forms.TextInput):
    class Media:
        css = {'all': ['javascript/library/select2-3.5.2/select2.css',
                       'javascript/library/select2-3.5.2/select2-bootstrap.css']}
        js = ['javascript/library/select2-3.5.2/select2.min.js',
              'javascript/library/select2-3.5.2/select2_locale_zh-CN.js']

    key_value_splitter = '`~`'

    def __init__(self, allow_create_tag=True, suggestion_url=None, attrs=None):
        super(TagAutocompleteInput, self).__init__(attrs=attrs)
        self.allow_create_tag = allow_create_tag
        self.suggestion_url = suggestion_url

    def render(self, name, value, attrs=None):
        """
        :param name: 控件名字
        :param value: 默认值. 可以采用 v1_:_text1,v2_:_text2 ...形式
        :param attrs: 附加的html属性
        """
        create_new_tag = self.allow_create_tag
        suggestion_url = resolve_url(self.suggestion_url if self.suggestion_url else 'tag_suggestion')
        js = u"""<script>
        $(document).ready(function(){{
        $("#{id_for_label}").select2({{
        placeholder: '{placeholder}',
            tags:{tag_model},
            multiple: true,
            tokenSeparators:{tag_model}?[",", " ",";"]:[],
            ajax: {{
                url: "{url}",
                dataType: 'json',
                data: function (term) {{
                    return{{term: term}}
                }},
                results: function (data) {{
                    return {{results: data}};
                }}
            }},
            {create_choice}
            initSelection: function (element, callback) {{
                var data = [],vals=[];
                $(element.val().split(",")).each(function (index,name) {{
                var kv=name.split("{splitter}");
                if (kv.length==1){{
                    data.push({{id: this, text: name}});
                    vals.push(this);
                }}
                else{{
                    data.push({{id:kv[0], text: kv[1]}});
                    vals.push(kv[0]);
                }}
                }});
                callback(data);
                element.val(vals.toString());
            }},
            minimumInputLength: 1,
            triggerChange: true
        }})
        }})
        </script>
        """
        js = js.format(id_for_label=force_text(self.id_for_label('id_%s' % name)),
                       url=force_text(suggestion_url),
                       placeholder=attrs.get('placeholder', '') if attrs else '',
                       tag_model='true' if self.allow_create_tag else 'false',
                       create_choice=""" createSearchChoice: function(term, data) {
                if ($(data).filter(function() {
                  return this.text == undefined || this.text.localeCompare(term) === 0;
                }).length === 0) {
                  return {
                    id: term,
                    text: term
                  };
                }
           },""" if create_new_tag else "",
                       splitter=self.key_value_splitter
                       )

        code = "%s %s" % (super(TagAutocompleteInput, self).render(name, value, attrs), js)
        return mark_safe(code)

    def value_from_datadict(self, data, files, name):
        value = data.get(name)
        if value and self.key_value_splitter in value:
            result = []
            for item in value.split(','):
                v = item.split(self.key_value_splitter)[0]
                if v not in result:
                    result.append(v)
            value = u','.join(result)
        return value



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings

try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module


# Default settings
ZUI_DEFAULTS = {
    'jquery_url': settings.STATIC_URL + "zui/lib/jquery/jquery.js",
    'base_url': settings.STATIC_URL + "zui/",
    'css_url': None,
    'theme_url': None,
    'javascript_url': None,
    'javascript_in_head': False,
    'include_jquery': False,
    'horizontal_label_class': 'col-md-3',
    'horizontal_field_class': 'col-md-9',
    'set_required': True,
    'set_disabled': False,
    'set_placeholder': True,
    'required_css_class': '',
    'error_css_class': 'has-error',
    'success_css_class': 'has-success',
    'formset_renderers': {
        'default': 'scaffold_toolkit.zui.renderers.FormsetRenderer',
    },
    'form_renderers': {
        'default': 'scaffold_toolkit.zui.renderers.FormRenderer',
    },
    'field_renderers': {
        'default': 'scaffold_toolkit.zui.renderers.FieldRenderer',
        'inline': 'scaffold_toolkit.zui.renderers.InlineFieldRenderer',
    },
    'model_renderers': {
        'default': 'scaffold_toolkit.zui.model_renderers.ModelRenderer',
    },
    'model_field_renderers': {
        'default': 'scaffold_toolkit.zui.model_renderers.ModelFieldRenderer'
    },
    'help_text_as_placeholder': True
}

# Start with a copy of default settings
ZUI = ZUI_DEFAULTS.copy()

# Override with user settings from settings.py
ZUI.update(getattr(settings, 'ZUI', {}))


def get_zui_setting(setting, default=None):
    """
    Read a setting
    """
    return ZUI.get(setting, default)


def zui_url(postfix):
    """
    Prefix a relative url with the zui base url
    """
    return get_zui_setting('base_url') + postfix


def jquery_url():
    """
    Return the full url to jQuery file to use
    """
    return get_zui_setting('jquery_url')


def javascript_url():
    """
    Return the full url to the Bootstrap JavaScript file
    """
    return get_zui_setting('javascript_url') or \
           zui_url('js/zui.min.js')


def css_url():
    """
    Return the full url to the Bootstrap CSS file
    """
    return get_zui_setting('css_url') or \
           zui_url('css/zui.min.css')


def theme_url():
    """
    Return the full url to the theme CSS file
    """
    return get_zui_setting('theme_url')


def get_renderer(renderers, **kwargs):
    layout = kwargs.get('layout', '')
    path = renderers.get(layout, renderers['default'])
    mod, cls = path.rsplit(".", 1)
    return getattr(import_module(mod), cls)


def get_formset_renderer(**kwargs):
    renderers = get_zui_setting('formset_renderers')
    return get_renderer(renderers, **kwargs)


def get_form_renderer(**kwargs):
    renderers = get_zui_setting('form_renderers')
    return get_renderer(renderers, **kwargs)


def get_field_renderer(**kwargs):
    renderers = get_zui_setting('field_renderers')
    return get_renderer(renderers, **kwargs)


def get_model_renderer(**kwargs):
    renderers = get_zui_setting('model_renderers')
    return get_renderer(renderers, **kwargs)


def get_model_field_renderer(**kwargs):
    renderers = get_zui_setting('model_field_renderers')
    return get_renderer(renderers, **kwargs)



# coding=utf-8

# 自定义的model renders
from django.db.models import Model, AutoField
from django.template import Template
from django.template import Context
from django.utils.safestring import mark_safe
from .forms import (
    render_form_group
)
from .renderers import BaseRenderer
from .exceptions import BootstrapError
from .utils import add_css_class
from .zui import get_model_field_renderer, get_model_renderer
from .text import text_value


def render_model(object, **kwargs):
    renderer_cls = get_model_renderer(**kwargs)
    return renderer_cls(object, **kwargs).render()


def render_model_field(object, field, **kwargs):
    """
    Render a model to Zui layout
    :param field:
    :param kwargs:
    :return:
    """
    renderer_cls = get_model_field_renderer(**kwargs)
    return renderer_cls(object, field, **kwargs).render()


class ModelRenderer(BaseRenderer):
    """
    Default form renderer
    """

    def __init__(self, object, *args, **kwargs):
        if not isinstance(object, Model):
            raise BootstrapError(
                'Parameter "object" should contain a valid Django Form.')
        self.object = object
        super(ModelRenderer, self).__init__(*args, **kwargs)
        # Handle form.empty_permitted
        # if self.form.empty_permitted:
        # self.set_required = False

    def render_fields(self):
        rendered_fields = []

        for field in self.object._meta.fields:
            rendered_fields.append(render_model_field(self.object,
                                                      field,
                                                      layout=self.layout,
                                                      form_group_class=self.form_group_class,
                                                      field_class=self.field_class,
                                                      label_class=self.label_class,
                                                      show_help=self.show_help,
                                                      exclude=self.exclude,
                                                      set_required=self.set_required,
                                                      size=self.size,
                                                      horizontal_label_class=self.horizontal_label_class,
                                                      horizontal_field_class=self.horizontal_field_class,
                                                      ))
        return '\n'.join(rendered_fields)

    def render(self):
        return self.render_fields()


class ModelFieldRenderer(BaseRenderer):
    def __init__(self, object, field, *args, **kwargs):
        self.object = object
        self.field = field
        super(ModelFieldRenderer, self).__init__(*args, **kwargs)
        self.field_help = text_value(mark_safe(field.help_text)) \
            if self.show_help and field.help_text else ''

    def append_to_field(self, html):
        help_text_and_errors = [self.field_help]
        if help_text_and_errors:
            help_html = Template(
                "{{ help_text_and_errors|join:' ' }}"
            ).render(Context({
                'field': self.field,
                'help_text_and_errors': help_text_and_errors,
                'layout': self.layout,
            }))
            html += '<span class="help-block">{help}</span>'.format(
                help=help_html)
        return html

    def get_field_class(self):
        field_class = self.field_class
        if not field_class and self.layout == 'horizontal':
            field_class = self.horizontal_field_class
        return field_class

    def wrap_field(self, html):
        field_class = self.get_field_class()
        if field_class:
            html = '<div class="{klass}"><p class="form-control-static">{html}</p></div>'.format(
                klass=field_class, html=html)
        return html

    def get_label_class(self):
        label_class = self.label_class
        if not label_class and self.layout == 'horizontal':
            label_class = add_css_class(self.horizontal_label_class, 'control-label')
        if self.layout == 'inline':
            label_class = 'sr-only'
        label_class = text_value(label_class)
        if not self.show_label:
            label_class = add_css_class(label_class, 'sr-only')
        return label_class

    def get_label(self):
        label = self.field.verbose_name
        if self.layout == 'horizontal' and not label:
            return '&#160;'
        return label

    def add_label(self, html):
        label = self.get_label()
        if label:
            html = """<label class="{label_class}">{label}</label>""".format(label_class=self.get_label_class(),
                                                                             label=label) + html
        return html

    def get_form_group_class(self):
        form_group_class = self.form_group_class

        if self.layout == 'horizontal':
            form_group_class = add_css_class(
                form_group_class, self.get_size_class(prefix='form-group'))
        return form_group_class

    def wrap_label_and_field(self, html):
        return render_form_group(html, self.get_form_group_class())

    def render(self):
        # See if we're not excluded
        if self.field.name in self.exclude.replace(' ', '').split(','):
            return ''
        # Hidden input requires no special treatment
        if isinstance(self.field, AutoField):
            return text_value(getattr(self.object, self.field.name))

        html = text_value(getattr(self.object, self.field.name))

        # Start post render
        html = self.append_to_field(html)
        html = self.wrap_field(html)
        html = self.add_label(html)
        html = self.wrap_label_and_field(html)
        return html



# coding=utf-8
import json
from django import template
from django.conf import settings
from django.forms import forms
from django.forms import fields
from django.utils.html import format_html
from django.core.validators import MinLengthValidator, MaxLengthValidator, MinValueValidator, MaxValueValidator
from django.utils.safestring import mark_safe
from scaffold_toolkit.formvalidator.forms.validators import BaseBV, ImageFileValidator
from .utils import convert_datetime_python_to_javascript, get_language

register = template.Library()


def _get_static_url(path):
    from django.contrib.staticfiles.storage import staticfiles_storage

    return staticfiles_storage.url(path)


@register.simple_tag
def formvalidator_javascript(framework='bootstrap', language=None):
    language = get_language() if language is None else language
    language = 'zh_CN' if language == 'zh_HANS' else language
    return format_html('{base}{framework}{language}',
            base=format_html('<script src="{url}"></script>', url=_get_static_url('formvalidator/js/formValidation.min.js')),
            framework=format_html('<script src="{url}"></script>',
                    url=_get_static_url('formvalidator/js/framework/{}.min.js'.format(framework))),
            language=format_html('<script src="{url}"></script>',
                    url=_get_static_url('formvalidator/js/language/{}.js'.format(language)))
    )


@register.simple_tag
def formvalidator_css():
    return format_html('<link href="{url}" rel="stylesheet" />',
            url=_get_static_url('formvalidator/css/formValidation.min.css'))


@register.simple_tag
def formvalidator(selector, form, requirejs=False, *args, **kwargs):
    """

    :param selector:
    :type selector: str
    :param form:
     :type form: django.forms.Form
    :param requirejs:
    :param args:
    :param kwargs:(language,excluded)
    :return:
    """
    if not selector.startswith(u'.') and not selector.startswith('#'):
        selector = '#' + selector
    container = kwargs.pop('err.container', '') or kwargs.pop('container', '')
    icon = kwargs.pop('icon', None)
    excluded = kwargs.pop('excluded', ':disabled, :hidden, :not(:visible)')

    validators = {}
    for field in form:
        validators[field.name] = render_field(field)
    code = (u"$(document).ready(function() {{ \r\n"
            u"      $('{selector}').formValidation({{  \r\n"
            u"          framework: 'bootstrap',  \r\n"
            u"          err:{{container:'{container}'}},  \r\n"
            u"          icon: {icon},  \r\n"
            u"          locale:'{lang}',  \r\n"
            u"          excluded:'{excluded}',  \r\n"
            u"          fields:{fields} \r\n"
            u"      }}) \r\n"
            u'  }});')
    icon = icon.lower() if icon else None
    if icon == 'fa' or icon == 'fontawesome':
        icon_code = (u" { \r\n"
                     u"valid: 'fa fa-check', \r\n"
                     u"invalid: 'fa fa-times', \r\n"
                     u"validating: 'fa fa-refresh' \r\n"
                     u"} \r\n")
    elif icon == 'bootstrap2' or icon == 'zui' or icon == 'buildin':
        icon_code = (u" { \r\n"
                     u"valid: 'icon-ok', \r\n"
                     u"invalid: 'icon-remove', \r\n"
                     u"validating: 'icon-refresh icon-spin' \r\n"
                     u"} \r\n")
    elif icon:
        icon_code = icon
    else:
        icon_code = 'null'
    language = get_language()
    if language == 'zh_HANS':
        language = 'zh_CN'
    vld_code = code.format(selector=selector, container=container, icon=icon_code,
                           excluded=excluded,
                           fields=json.dumps(validators, indent=4), lang=language)
    if requirejs:
        depends = '"jquery","formValidator"'
        language = kwargs.pop('language', get_language())
        depends = '{},"formValidator/language/{}"'.format(depends, language)
        vld_code = u'requirejs([{}],function(){{ {} }})'.format(depends, vld_code)

        return mark_safe("<script>{}</script>".format(get_require_config_code() + vld_code))
    else:
        return mark_safe("<script>{}</script>".format(vld_code))


@register.simple_tag
def formvalidator_fields(form):
    validators = {}
    for field in form:
        validators[field.name] = render_field(field)
    return mark_safe(json.dumps(validators, indent=4))


@register.simple_tag
def formvalidator_requirejs_config(base_url=None, language=None):
    return mark_safe(get_require_config_code(base_url, language))


def get_require_config_code(base_url=None, language=None):
    config = ("""

    if (!require.defined("formValidator")){{
        require.config({{
              paths:{{
              'formValidator':'{bv}/formValidator',
              '_formValidation':'{bv}/js/formValidation.min',
              '_formValidation/framework/bootstrap':'{bv}/js/framework/bootstrap.min',
              'formValidator/language/{lang}':'{bv}/js/language/{lang}',
              }},
              shim:{{
              '_formValidation':['jquery'],
               '_formValidation/framework/bootstrap':['_formValidation'],
              'formValidator':['_formValidation','_formValidation/framework/bootstrap'],
              'formValidator/language/{lang}':['formValidator']
              }}
        }});
    }}
    """)
    if base_url is None:
        bv = getattr(settings, 'FORM_VALIDATOR_PREFIX', '')
        if not bv:
            bv = _get_static_url('formvalidator')
    else:
        bv = base_url
    if bv.endswith("/"):
        bv = bv[:-1]
    language = language if language else get_language()

    return config.format(bv=bv, lang=language)


def render_field(field):
    """
    渲染字段验证代码
    :param field:
     :type field: django.forms.Field
    :return:
    """
    field = field.field if isinstance(field, forms.BoundField) else field
    validators = {}

    def no_compare_validator():
        return not ('lessThan' in validators or 'greaterThan' in validators or 'between' in validators)

    if field.required:
        validators['notEmpty'] = {}
    validator_codes = [item.code for item in field.validators]
    for v in field.validators:
        if isinstance(v, MinLengthValidator):
            vc = validators.get('stringLength', {})
            vc['min'] = field.min_length
            validators.update({'stringLength': vc})
        elif isinstance(v, MaxLengthValidator):
            vc = validators.get('stringLength', {})
            vc['max'] = field.max_length
            validators.update({'stringLength': vc})
        elif isinstance(v, (MinValueValidator, MaxValueValidator)):
            if 'min_value' in validator_codes and 'max_value' in validator_codes:
                vc = validators.get('between', {})
                if v.code == 'min_value':
                    vc['min'] = field.min_value
                else:
                    vc['max'] = field.max_value
                validators.update({'between': vc})
            elif v.code == 'min_value':
                validators['greaterThan'] = {'value': field.min_value}
            elif v.code == 'max_value':
                validators['lessThan'] = {'value': field.max_value}
        elif isinstance(v, BaseBV):
            validators.update(v.get_validator_code())

    if isinstance(field, (fields.DecimalField, fields.FloatField)) and no_compare_validator():
        validators['numeric'] = {}
    elif isinstance(field, fields.IntegerField) and no_compare_validator():
        validators['integer'] = {}
    elif isinstance(field, (fields.DateField, fields.DateTimeField)):
        formats = field.input_formats
        if formats:
            validators['date'] = {'format': convert_datetime_python_to_javascript(formats[0])}
    elif isinstance(field, fields.TimeField):
        validators['regexp'] = {'regexp': '^((([0-1]?[0-9])|([2][0-3])):)(([0-5][0-9]):)([0-5][0-9])$',
                                }
    elif isinstance(field, fields.URLField):
        validators['uri'] = {}
    elif isinstance(field, fields.EmailField):
        validators['emailAddress'] = {}
    elif isinstance(field, fields.ImageField):
        if 'file' not in validators:
            validators.update(ImageFileValidator().get_validator_code())
    return {'validators': validators}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from django.forms.widgets import flatatt
except:
    from django.forms.utils import flatatt

from .text import text_value


def render_icon(icon, title=''):
    """
    Render a zui  icon
    """
    attrs = {
        'class': 'icon-{icon}'.format(icon=icon),
    }
    if title:
        attrs['title'] = title
    return '<span{attrs}></span>'.format(attrs=flatatt(attrs))


def render_alert(content, alert_type=None, dismissable=True):
    """
    Render a zui alert
    """
    button = ''
    if not alert_type:
        alert_type = 'info'
    css_classes = ['alert', 'alert-' + text_value(alert_type)]
    if dismissable:
        css_classes.append('alert-dismissable')
        button = '<button type="button" class="close" ' + \
                 'data-dismiss="alert" aria-hidden="true">&times;</button>'
    return '<div class="{css_classes}">{button}{content}</div>'.format(
        css_classes=' '.join(css_classes),
        button=button,
        content=text_value(content),
    )



# coding=utf-8
import codecs

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import os
from setuptools import find_packages

base_dir = os.path.dirname(__file__)
setup(
    name='django-scaffold-toolkit',
    version='0.5.2',
    author='YuanXu',
    author_email='ankh2008@gmail.com',
    packages=['scaffold_toolkit'] + ['scaffold_toolkit.%s' % item for item in find_packages("scaffold_toolkit")],
    url="https://github.com/yuanxu/django-scaffold",
    license='LICENSE.txt',
    description=u'Django开发中一组常用工具。包括表单生成，验证，富文本编辑器，useragent检测等等',
    include_package_data=True,
    long_description=codecs.open(os.path.join(base_dir, 'README.md'), encoding='utf-8').read(),
    install_requires=[],  # ['django', 'requests', 'django-braces', 'shortuuid'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
    ],
)
